searchData={"items":[{"type":"module","title":"Sibyl","doc":"Sibyl is a library which augments the BEAM's default tracing capabilities by hooking\ninto `:telemetry`, `:dbg` (the BEAM's built in tracing and debugging functionality),\nand `OpenTelemetry`.","ref":"Sibyl.html"},{"type":"module","title":"Basic Usage - Sibyl","doc":"To leverage all that Sibyl gives you, you need to use it in a module like so:\n\n```elixir\ndefmodule MyApp.Users do\n  use Sibyl\nend\n```\n\nFollowing this, you're able to begin emitting telemetry events and tracing function\nexecution.","ref":"Sibyl.html#module-basic-usage"},{"type":"module","title":"Tracing Function Execution - Sibyl","doc":"Sibyl provides two decorators which you can use in your modules to automatically\ntrace function execution. These are `@decorate_all trace()` and `@decorate trace()`,\nwhich automatically traces _all_ functions in the given module, or the function\nmost immediately following the decorator respectively.\n\nSibyl's function tracing follows `:telemetry`'s standard specification for capturing\nspans. At the beginning of a function a `:start` event is emitted; and at the end\nof a function a `:stop` event is emitted. If an exception (arising from a `raise`\nor `throw`) is detected, an `:exception` event is emitted instead.\n\nUnlike `:telemetry.span/3` however, Sibyl's trace decorators inline these event\nemissions into the compiled bytecode of your module, which is slightly more efficient\nthan wrapping traces within anonymous functions. This also has the benefit of making\nstacktraces easier to read.\n\nEvent names are automatically determined such that a `:start` event is emitted by\nthe function `MyApp.Users.sign_up`; the `[:my_app, :users, :sign_up, :start]` event\nis emitted.\n\nExamples follow:\n\n```elixir\ndefmodule MyApp.Users do\n  use Sibyl\n\n  @decorate trace()\n  def sign_up(attrs) do\n    :ok\n  end\nend\n\ndefmodule MyApp.Mailer do\n  use Sibyl\n\n  @decorate_all trace()\n\n  def build_mail(attrs) do\n    :ok\n  end\n\n  def send_mail(attrs) do\n    attrs\n    |> build_mail()\n    |> handle_send()\n  end\nend\n```","ref":"Sibyl.html#module-tracing-function-execution"},{"type":"module","title":"Custom Event Emission - Sibyl","doc":"Sibyl also provides a thin wrapper over `:telemetry.execute/3` to make event emission\nless error prone.\n\nSibyl will perform compile time checks prior to attempting to emit an event to make\nsure that it has previously been defined in the current module.\n\nEvents which are defined in a module are automatically prefixed with that module's\nnamespace such that given a module `MyApp.Users` and an event `:registered`, the\nresultant event will be compiled and ultimately emitted as `[:my_app, :users, :registered]`.\n\nOtherwise, you can specify a module which defined a given event by using `Sibyl.emit/4` over\n`Sibyl.emit/3`.\n\nFor more in-depth information, see `Sibyl.emit/4`.\n\nExamples follow:\n\n```elixir\ndefmodule MyApp.Users do\n  use Sibyl\n\n  def sign_up(attrs) do\n    emit :registered  # Fails to compile as event is unknown.\n  end\nend\n\ndefmodule MyApp.Users do\n  use Sibyl\n\n  define_event :registered\n\n  def sign_up(attrs) do\n    emit :registered  # Compiles properly and emits event\n  end\nend\n\ndefmodule MyApp.Users.Mailer do\n  use Sibyl\n\n  def send_email do\n    emit MyApp.Users, :registered    # Compiles properly and emits events\n    emit MyApp.Users, :invalid_event # Causes error as this was not defined in specified module\n  end\nend\n```","ref":"Sibyl.html#module-custom-event-emission"},{"type":"module","title":"Reflection - Sibyl","doc":"Due to the fact that Sibyl is able to check whether or not events have been defined\nprior to use, Sibyl exposes a reflection API in the way of `Sibyl.Events.reflect/0`\nand `Sibyl.Events.reflect/1`.\n\nPlease see documentation for `Sibyl.Events` if you're interested in more potential\navenues for extending Sibyl or metaprogramming.","ref":"Sibyl.html#module-reflection"},{"type":"module","title":"Telemetry Handlers - Sibyl","doc":"Building on top of Sibyl's reflection API, we are able to provide functions to\nautomatically attach defined events to `:telemetry` handlers. You can do this via\nthe helper functions in `Sibyl.Handlers`.\n\nPlease see the documentation for `Sibyl.Handlers` for more information, but a brief\nusage example follows:\n\n```elixir\n@impl Sibyl.Handler\ndef handle_event(event, measurement, metadata, opts) do\n  IO.inspect({event, measurement, metadata, opts})\nend\n\n:ok = Sibyl.Handlers.attach_all_events(__MODULE__)\n```\n\nAdditionally, Sibyl provides two example `:telemetry` handlers: a very basic\nElixir `Logger` handler for demonstration purposes, as well as an `OpenTelemetry`\nhandler which was what prompted the building of Sibyl in the first place.","ref":"Sibyl.html#module-telemetry-handlers"},{"type":"module","title":"OpenTelemetry Integration - Sibyl","doc":"OpenTelemetry is a widely understood specification for dealing with event emission\nand traces.\n\nOne can use `OpenTelemetry` to be able to instrument your code base with events and\ntraces quite easily. However; the Elixir community also very much utilises `:telemetry`\nas the standard telemetry/metric/span gathering library.\n\nIt is possible, of course, to use `OpenTelemetry` as well as `:telemetry`, but it\nwould be convinient to have one unified API which bridges both worlds.\n\nVery much inspired by the `OpentelemetryTelemetry` library, Sibyl provides a generic\n`:telemetry` handler which bridges *any* `:telemetry.span/3`-spec complaint events\nto `OpenTelemetry` traces.\n\nSimply attach the `Sibyl.Handlers.OpenTelemetry` handler and start tracing functions,\nany captured traces will be handled by your configured OLTP exporter of choice.\n\nFor demonstration purposes, this project also contains a `docker-compose.yml` which\nsets up `Jaeger`: an easy to use distributed tracing UI to view spans which understands\n`OpenTelemetry`.","ref":"Sibyl.html#module-opentelemetry-integration"},{"type":"module","title":"Dynamic Tracing - Sibyl","doc":"In order to aide debugging of running systems without needing to instrument your code\nwith decorators or event emission, Sibyl also provides an *experimental* dynamic\ntracer which leverages the BEAM's built in tracing functionalities.\n\nPlease make sure you understand what turning on the BEAM's tracing functionality\ncan do to overload a production system however. This is not neccessarily advisable,\nbut it _is_ possible.\n\nAfter enabling Sibyl's dynamic tracer and attaching a `:telemetry` handler, all future\ninvokations of any given functions will be handled as though you had instrumented\nyour codebase with `@decorate trace()`.\n\nExamples follow:\n\n```elixir\n{:ok, _meta} = Sibyl.Dynamic.enable(Sibyl.Handlers.OpenTelemetry)\n{:ok, _meta} = Sibyl.Dynamic.trace(Enum, :map, 2)\n\nEnum.map([1, 2, 3], & &1) # Produces OpenTelemetry traces\n\n:ok = Sibyl.Dynamic.disable()\n```","ref":"Sibyl.html#module-dynamic-tracing"},{"type":"macro","title":"Sibyl.emit/4","doc":"Emits an event.\n\nCan be called in one of two modes, either: `Sibyl.emit/4` or `Sibyl.emit/3`.\n\nWhen called via the arity-4 variant, expects the following arguments to be passed in:\n1) A module alias such as `Enum` or `MyApp.Telemetry`.\n2) An atom: the name of the event you wish to emit, which should be defined in the given module.\n3) An optional map containing \"measurements\" for the given event.\n4) An optional map containing \"metadata\" for the given event.\n\nWhen called via the arity-3 variant, expects the following arguments to be passed in:\n1) An event, either in the form of a singular atom, or in the form of a list of atoms.\n  - If a singular atom is passed in, it is expected that the given atom is defined as\n    as event in the caller's module.\n  - If a list of atoms is passed in, no expectations hold as `Sibyl` assumes you're\n    trying to emit an event that _was not_ defined by `Sibyl`.\n2) An optional map containing \"measurements\" for the given event.\n3) An optional map containing \"metadata\" for the given event.\n\nNo other combination of arguments is supported and an error will be raised at compile time\nif called any other way than described.\n\nWhen called properly, `Sibyl` will perform a compile-time check on the event you're attempting\nto emit (unless the event was specified as a list of atoms), to ensure that the event was\nproperly defined by `Sibyl` before use.","ref":"Sibyl.html#emit/4"},{"type":"module","title":"Sibyl.AST","doc":"Utility module for working with ASTs","ref":"Sibyl.AST.html"},{"type":"type","title":"Sibyl.AST.alias/0","doc":"","ref":"Sibyl.AST.html#t:alias/0"},{"type":"macro","title":"Sibyl.AST.alias?/1","doc":"Returns true if the given argument is an Elixir AST node representing a module alias\nsuch as `Enum`.","ref":"Sibyl.AST.html#alias?/1"},{"type":"type","title":"Sibyl.AST.ast/0","doc":"","ref":"Sibyl.AST.html#t:ast/0"},{"type":"function","title":"Sibyl.AST.module/2","doc":"Given an alias AST node, returns the fully resolved alias that said node would expand\nto.\n\nFor example, given: `{:__aliases, unused(), [Elixir, Enum]}`, returns: `Enum`.","ref":"Sibyl.AST.html#module/2"},{"type":"macro","title":"Sibyl.AST.unused/0","doc":"Returns the `:__unused__` atom.","ref":"Sibyl.AST.html#unused/0"},{"type":"type","title":"Sibyl.AST.unused/0","doc":"","ref":"Sibyl.AST.html#t:unused/0"},{"type":"macro","title":"Sibyl.AST.unused?/1","doc":"Returns true if the given argument is equal to `:__unused__`. Primarily used internally.","ref":"Sibyl.AST.html#unused?/1"},{"type":"module","title":"Sibyl.Decorator","doc":"Module encapsulating Sibyl's business logic for decorating functions for automated\ntracing.\n\nShould be used only via `use Sibyl`.\n\nWhen used like this, provides the ability to decorate your function calls with:\n\n- `@decorate trace()` to automatically trace a single function's execution, emitting\n  telemetry events for `:start`, `:end`, and `:exception` sub-events.\n\n- `@decorate_all trace()` which does the same as the above, but automatically for\n  all functions in a module.\n\nAutomatically traced functions are available for reflection by via `Sibyl.Events.reflect/1`.","ref":"Sibyl.Decorator.html"},{"type":"macro","title":"Sibyl.Decorator.trace/0","doc":"","ref":"Sibyl.Decorator.html#trace/0"},{"type":"function","title":"Sibyl.Decorator.trace/2","doc":"Decorator which wraps a given function with a standard telemetry span.\n\nThe name of the captured event will be determined by however `Sibyl` is configured\nto generate event names.\n\nDue to how anonymous functions are defined and executed in the BEAM, it ends up\nbeing quite a bit more performant to build the span manually rather than using\n`:telemetry.span/3`.\n\nSee [here](https://keathley.io/blog/telemetry-conventions.html) for an example of\nhow to emit the correct events.\n\nSee [here](https://github.com/beam-telemetry/telemetry/pull/43) for explanations\nw.r.t. anonymous function perf.","ref":"Sibyl.Decorator.html#trace/2"},{"type":"module","title":"Sibyl.Dynamic","doc":"Module which contains functions which allow you to bridge together the built in\nBEAM debugging and tracer with modules implementing the `Sibyl.Handler` behaviour.\n\nThis provides extremely powerful functionality as you can effectively instruct\nthe BEAM to start building OpenTelemetry traces without code instrumentation on\nproduction environments.\n\nIt is important to note that leveraging the BEAM's built in debugging and tracing\nfunctionality can have severe memory and CPU requirements so it isn't something\none should do lightly.\n\nRegardless, in order to track down heisenbugs and other maladies which can only\nbe reproduced on production environments, this functionality should prove extremely\nuseful.","ref":"Sibyl.Dynamic.html"},{"type":"function","title":"Sibyl.Dynamic.disable/0","doc":"","ref":"Sibyl.Dynamic.html#disable/0"},{"type":"function","title":"Sibyl.Dynamic.enable/1","doc":"","ref":"Sibyl.Dynamic.html#enable/1"},{"type":"function","title":"Sibyl.Dynamic.trace/3","doc":"","ref":"Sibyl.Dynamic.html#trace/3"},{"type":"module","title":"Sibyl.Dynamic.Guards","doc":"Utils module contianing various custom guards in the scope of `Sibyl.Dynamic`","ref":"Sibyl.Dynamic.Guards.html"},{"type":"macro","title":"Sibyl.Dynamic.Guards.is_trace/1","doc":"Returns `true` if given term is a message sent by the Erlang `:dbg` module for\nfunction and function-return traces.","ref":"Sibyl.Dynamic.Guards.html#is_trace/1"},{"type":"macro","title":"Sibyl.Dynamic.Guards.is_type/2","doc":"Returns `true` if given term is a message sent by the Erlang `:dbg` module and if\nthe message type corresponds to the given type.","ref":"Sibyl.Dynamic.Guards.html#is_type/2"},{"type":"module","title":"Sibyl.Events","doc":"Module containing the core business logic of Sibyl.\n\nIncludes utility functions to defining events, emitting events, and reflecting\non events which are defined in any module loaded on the BEAM.","ref":"Sibyl.Events.html"},{"type":"macro","title":"Sibyl.Events.build_event/1","doc":"Given an atom denoting some abstract event (but not of type `event()`), builds an event\nname such that the given abstract event is formatted as per telemetry best practices.","ref":"Sibyl.Events.html#build_event/1"},{"type":"macro","title":"Examples (assuming this is being called in a module and function) - Sibyl.Events.build_event/1","doc":"iex> Sibyl.Events.build_event(:not_found)\n    [:my_app, :\"some_function/2\", :not_found]","ref":"Sibyl.Events.html#build_event/1-examples-assuming-this-is-being-called-in-a-module-and-function"},{"type":"function","title":"Sibyl.Events.build_event/4","doc":"Builds a consistent telemetry event name following the conventions specified in\n[this post](https://keathley.io/blog/telemetry-conventions.html).\n\nCalled like: `Sibyl.build_event(MyApp.Accounts, :register_user, 2, :email_sent)`,\nproduces the following event: `[:my_app, :accounts, :\"register_user/2\", :email_sent]`","ref":"Sibyl.Events.html#build_event/4"},{"type":"macro","title":"Sibyl.Events.define_event/2","doc":"Defines the given event.\n\nEvents should only be used once they are defined. Unless this is done, `reflect/0`\nand `reflect/1` will fail to see said event and will eventually throw errors.\n\nWhen given a singular atom, it is assumed that the event you are trying to define\nis namespaced to said module as: `[:my_app, :my_module, :custom_event]`.\n\nWhen given a list of atoms, the event is simply registered as whatever you passed\nin; i.e. `[:some, :custom, :event]`.","ref":"Sibyl.Events.html#define_event/2"},{"type":"function","title":"Sibyl.Events.emit/3","doc":"See `emit/4`","ref":"Sibyl.Events.html#emit/3"},{"type":"function","title":"Sibyl.Events.emit/4","doc":"Emits the given event.\n\nNote: this is a low level API which should not be called outside of Sibyl's own\ncode, and largely exists to plumb together the system.\n\nPlease prefer to use `Sibyl.emit/3` which includes compile time checks to make\nsure the events being emitted were registered.\n\nWhen given an event as an atom, tries to emit the event namespaced to the given\nmodule (see `define_event/2` for more information).\n\nWhen given a event as a list, simply emits that given list.","ref":"Sibyl.Events.html#emit/4"},{"type":"type","title":"Sibyl.Events.event/0","doc":"","ref":"Sibyl.Events.html#t:event/0"},{"type":"function","title":"Sibyl.Events.event?/1","doc":"Returns true if event was defined.","ref":"Sibyl.Events.html#event?/1"},{"type":"function","title":"Sibyl.Events.event?/2","doc":"Given a module that may, or may not, `use Sibyl` as well as an event, returns\ntrue if said module defines the given event.","ref":"Sibyl.Events.html#event?/2"},{"type":"function","title":"Sibyl.Events.reflect/0","doc":"Returns a list of all telemetry events which have been defined by `Sibyl.Events`","ref":"Sibyl.Events.html#reflect/0"},{"type":"function","title":"Sibyl.Events.reflect/1","doc":"Returns a list of all telemetry events which have been defined by `Sibyl.Events` for\nthe given module.\n\nNote: some events may be implicitly defined via the top level `@decorate trace()`\nor `@decorate_all trace()` decorators.","ref":"Sibyl.Events.html#reflect/1"},{"type":"type","title":"Sibyl.Events.sibyl_event/0","doc":"","ref":"Sibyl.Events.html#t:sibyl_event/0"},{"type":"module","title":"Sibyl.Experimental","doc":"This is an experimental module for Sibyl which aims to replace the `use Sibyl` macro with something that\ndoes not rely on the existing `decorator` library in Elixir.\n\nThis is due to limitations in the `decorator` library which prevent us from being able to trace functions\nwhich define multiple function clauses with default paramters due to the way its implemented.\n\nThis module implements the bare neccessities to get Sibyl working without the `decorator` library, and as such,\ndoes not intend to replace the `decorator` library in general (though it could definitely be possible in the future).\n\nWe do this by replacing Elixir's `def` macro when defining functions to a custom one which checks if you've opted\ninto function tracing. If so, then we amend the AST of the function to include tracing code before passing it back\nto Elixir's `def` macro.\n\nThis is a very hacky way of doing things, but it seems to generally work.\n\nThere are two notable, core differences in using `Sibyl.Experimental` over `Sibyl` for the time being:\n\n1) In order to trace functions, you must annotate functions with `@sibyl trace: true`. In future, we may be able to\n   capture extra metadata to attach to traces via additional keyword parameters.\n\n2) In order to trace all functions in a module, you must `use Sibyl.Experimental, trace_all: true`. Theoretically\n   any metadata attached to individual functions should still be respected.\n\nFor reflection purposes, you can also use call `__traces__/0` on any module that uses `Sibyl.Experimental` though\nfor compatibility you may prefer to continue using `Sibyl.Events.reflect/1` instead.","ref":"Sibyl.Experimental.html"},{"type":"macro","title":"Sibyl.Experimental.def/2","doc":"","ref":"Sibyl.Experimental.html#def/2"},{"type":"behaviour","title":"Sibyl.Handler","doc":"Behaviour for `:telemetry` handlers","ref":"Sibyl.Handler.html"},{"type":"callback","title":"Sibyl.Handler.handle_event/4","doc":"","ref":"Sibyl.Handler.html#c:handle_event/4"},{"type":"module","title":"Sibyl.Handlers","doc":"Groups functions to make it easy to attach telemetry events to handlers","ref":"Sibyl.Handlers.html"},{"type":"function","title":"Sibyl.Handlers.attach_all_events/2","doc":"Reflects upon the state of the current application's modules and all other dynamically\nloaded module and attaches any events defined in those modules to the given handler.\n\nAny options are forwarded to the given handler also.\nTakes an optional, but recommended `:name => String.t()` option too.","ref":"Sibyl.Handlers.html#attach_all_events/2"},{"type":"function","title":"Sibyl.Handlers.attach_events/3","doc":"Attaches the given events the given handler.\n\nAny options are forwarded to the given handler also.\nTakes an optional, but recommended `:name => String.t()` option too.","ref":"Sibyl.Handlers.html#attach_events/3"},{"type":"function","title":"Sibyl.Handlers.attach_module_events/3","doc":"Reflects upon the given module and attaches any events defined in those modules to\nthe given handler.\n\nAny options are forwarded to the given handler also.\nTakes an optional, but recommended `:name => String.t()` option too.","ref":"Sibyl.Handlers.html#attach_module_events/3"},{"type":"type","title":"Sibyl.Handlers.handler/0","doc":"","ref":"Sibyl.Handlers.html#t:handler/0"},{"type":"module","title":"Sibyl.Handlers.FlameGraph","doc":"An example Telemetry handler converting `:telemetry` events into Chrome-compatible\nflamegraphs.\n\nExposes two additional functions when compared to traditional `Sibyl.Handler`s:\n\n1) `start/0` which instructs Elixir.Sibyl.Handlers.FlameGraph to start persisting metadata in order\n   to build the resultant flamegraph.\n\n2) `stop/1` which instructs Elixir.Sibyl.Handlers.FlameGraph to flush any captured metadata into a\n   JSON file for further use.\n\nIf you're using a Chrome-derived browser, you'll be able to open and introspect\ngenerated flamegraphs via the [Chrome Tracing](chrome://tracing) builtin. Otherwise,\nyou can use open source tools which understand Chrome Tracing's format to render\ngraphs.\n\nExamples of open source apps that can render Chrome Traces include\n[Speedscope](https://www.speedscope.app/) and [Perfetto](https://perfetto.dev/).\n\nNote that this utility has only been tested and confirmed working with telemetry\nexplicitly passed in via `Sibyl` and may not work when listening to arbitrary\n`:telemetry` events passed in from other OTP applications.\n\nNotes on the file format used by Google's tracer, Speedscope, and other related tools\n[can be found here](https://docs.google.com/document/d/1CvAClvFfyA5R-PhYUmn5OOQtYMH4h6I0nSsKchNAySU/edit#).","ref":"Sibyl.Handlers.FlameGraph.html"},{"type":"function","title":"Sibyl.Handlers.FlameGraph.start/0","doc":"Instructs Elixir.Sibyl.Handlers.FlameGraph to start capturing and persisting `:telemetry` metadata.","ref":"Sibyl.Handlers.FlameGraph.html#start/0"},{"type":"function","title":"Sibyl.Handlers.FlameGraph.started?/0","doc":"Returns `true` if Elixir.Sibyl.Handlers.FlameGraph has been started prior to invokation.","ref":"Sibyl.Handlers.FlameGraph.html#started?/0"},{"type":"function","title":"Sibyl.Handlers.FlameGraph.stop/1","doc":"Instructs Elixir.Sibyl.Handlers.FlameGraph to output any captured `:telemetry` metadata into the JSON\nfile of your choice.","ref":"Sibyl.Handlers.FlameGraph.html#stop/1"},{"type":"module","title":"Sibyl.Handlers.Logger","doc":"An example Telemetry handler for logging given events to the current application's\nconfigured logger.\n\nPlease note that this very much *is an example*. There is little reason to do this\nin production code.","ref":"Sibyl.Handlers.Logger.html"},{"type":"module","title":"Sibyl.Handlers.OpenTelemetry","doc":"OpenTelemetry is an open source standard telemetry standard which allows us to capture\ncustom metrics and traces of our application.\n\nFor local development, you can install tools such as [Jaeger](https://www.jaegertracing.io/)\nto be able to view and test OpenTelemetry traces.\n\nHowever, much of the BEAM ecosystem (and this library) uses `:telemetry` as a standard\nfor emitting arbitrary telemetry events.\n\nThis handler is a bridge between standard `:telemetry` span events and OpenTelemetry\nspec compliant traces.\n\n  - Any event which ends in `:start` will start an OpenTelemetry span context.\n\n  - Any event which ends in `:stop` or `:exception` will stop the currently active\n    OpenTelemetry span context, capturing any metadata that is passed in.\n\n  - Any event which ends in anything else will be attached as a custom event to\n    the currently active span context.\n\nFor Elixir's built in `Task` module, `Sibyl.Handlers.OpenTelemetry` will be able to\ninternal state and automatically attach said async Task to the original parent.\n\nDistributed traces via non `Task` means is also supported, but this cannot be automated.\nTo opt into this behaviour, serialize the span state on the consumer end via the\n`build_distributed_trace_context/0` function, and *prior* to starting any new traces\non the consumer end, use `attach_distributed_trace_context/1`. Doing this should\nattach both the consumer and producer side of the trace into one for rendering in tools\nsuch as Jaeger.","ref":"Sibyl.Handlers.OpenTelemetry.html"},{"type":"function","title":"Sibyl.Handlers.OpenTelemetry.attach_distributed_trace_context/1","doc":"Processes a serialized `OpenTelemetry` trace context (obtained via `build_distributed_trace_context/0`)\nand persists it within the current process.\n\nThe **next** span which is created will be automatically attached to this distributed\ntrace context, allowing for the building of (tooling permitting) distributed traces\nacross multiple different nodes, releases, services, etc.","ref":"Sibyl.Handlers.OpenTelemetry.html#attach_distributed_trace_context/1"},{"type":"function","title":"Sibyl.Handlers.OpenTelemetry.build_distributed_trace_context/0","doc":"Serializes the current `OpenTelemetry` trace context to allow it to be sent over-the-wire\nto external services, with the intention to re-attach the context on the consumer's end.\n\nThis should allow you (tooling permitting) to build distributed traces across multiple\ndifferent nodes, releases, services, etc.","ref":"Sibyl.Handlers.OpenTelemetry.html#build_distributed_trace_context/0"},{"type":"function","title":"Sibyl.Handlers.OpenTelemetry.undefined_trace_context/0","doc":"A constant for the undefined `OpenTelemetry` trace context.\n\nUseful for defaulting a distributed trace context parse to a \"noop\" if data is malformed\nor mistransmitted.","ref":"Sibyl.Handlers.OpenTelemetry.html#undefined_trace_context/0"},{"type":"behaviour","title":"Sibyl.Plugin","doc":"Open interface for defining custom Sibyl plugins.\n\nNote that at the time of writing, all plugins are considered in-development features and no\nstability or fixed API is guaranteed.\n\nSibyl Plugins are intended to be used by providing them as an option when attaching\nSibyl handlers like so:\n\n```\n:ok = Sibyl.Handlers.attach_all_events(plugins: [Sibyl.Plugins.Absinthe])\n:ok = Sibyl.Handlers.attach_module_events(MyApp.Module, plugins: [Sibyl.Plugins.Absinthe])\n:ok = Sibyl.Handlers.attach_events([[:my_app, :start]], plugins: [Sibyl.Plugins.Absinthe])\n```\n\nImplementing a plugin is done by defining a module which `use`s `Sibyl.Plugin` and implements\nthe behaviour below.\n\nPlease feel free to refer to `Sibyl.Plugins.Absinthe` or `Sibyl.Plugins.Ecto` for examples of\nhow to implement a plugin.","ref":"Sibyl.Plugin.html"},{"type":"callback","title":"Sibyl.Plugin.identity/0","doc":"Unique ID for the plugin. Intended to be used as a mechanism for attaching/detaching proxy `:telemetry` handlers","ref":"Sibyl.Plugin.html#c:identity/0"},{"type":"callback","title":"Sibyl.Plugin.init/1","doc":"Called by Sibyl when attaching a plugin to a handler. Should return a list of events which should be listened to by handlers","ref":"Sibyl.Plugin.html#c:init/1"},{"type":"callback","title":"Sibyl.Plugin.prefix/0","doc":"Event prefix for events emitted/proxied by a given plugin implementation. Defaults to the module's name split","ref":"Sibyl.Plugin.html#c:prefix/0"},{"type":"callback","title":"Sibyl.Plugin.stop/0","doc":"Stops and cleans up any resources used by the plugin. Intended primarily for `:telemetry` handler detachment","ref":"Sibyl.Plugin.html#c:stop/0"},{"type":"module","title":"Sibyl.Plugins.Absinthe","doc":"Sibyl plugin module for listening to telemetry events emitted by Absinthe. See docs for `Sibyl.Plugin`\nfor more information about plugins themselves.\n\nThis plugin will extend Sibyl and enable any configured Sibyl handler to listen to various events emitted\nby `Absinthe` including field resolution, query execution, and batching.\n\nUse via `Sibyl.Handlers.attach_all_events(plugins: [Sibyl.Plugins.Absinthe])`.\n\n> #### Note {: .warning}\n>\n> This plugin is very much still a work in progress, and should not be used in production code!\n>\n> Additionally, as this plugin is still a work in progress, it is not guaranteed to be stable. There\n> are no unit tests provided to ensure this module does not change.","ref":"Sibyl.Plugins.Absinthe.html"},{"type":"module","title":"Sibyl.Plugins.Ecto","doc":"Sibyl plugin module for listening to telemetry events emitted by Ecto. See docs for `Sibyl.Plugin`\nfor more information about plugins themselves.\n\nThis plugin will extend Sibyl and enable any configured Sibyl handler to listen to various events emitted\nby `Ecto` including field resolution, query execution, and batching.\n\nUse via `Sibyl.Handlers.attach_all_events(plugins: [Sibyl.Plugins.Ecto])`.\n\n> #### Note {: .warning}\n>\n> This plugin is very much still a work in progress, and should not be used in production code!\n> This is due to the fact that Sibyl doesn't expose an easy way to skew the time of events.\n> As a result, any spans containing Ecto functions will be very skewed and offset. Ecto spans alone\n> work fine however, so this is a workable first PoC\n>\n> Additionally, as this plugin is still a work in progress, it is not guaranteed to be stable. There\n> are no unit tests provided to ensure this module does not change.","ref":"Sibyl.Plugins.Ecto.html"},{"type":"exception","title":"Sibyl.BadEmissionError","doc":"","ref":"Sibyl.BadEmissionError.html"},{"type":"exception","title":"Sibyl.UndefinedEventError","doc":"","ref":"Sibyl.UndefinedEventError.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}